#!/bin/bash
#
# Copyright (c) 2009 Erno Rigo <erno@rigo.info>
# All rights reserved
#

cleanroot() {
    rm -rf "${TESTDIR}/root" 2>/dev/null 1>/dev/null
    mkdir "${TESTDIR}/root"
}

init() {
    export LANG=hu_HU.UTF-8
    export LC_ALL=hu_HU.UTF-8

    export LIBDIR=`dirname "$0"`
    export TESTDIR=`mktemp -d`
    trap "rm -rf $TESTDIR ;  exit 127" INT EXIT HUP

    . $LIBDIR/lib/colors.inc
    
    cat $LIBDIR/lib/req | while read cmd; do
	#echo "$cmd ellenőrzése..."
	res=`command -v $cmd`
	if [ "z$res" = "z" ]; then
	    echo "A következő parancs/program nem futtatható: '$cmd'"
	    echo "A gyakorló keretrendszer futtatásához szükséges, hogy a $LIBDIR/lib/req fájlban felsorolt összes parancs az ott megadott formában futtatható legyen!"
	    exit -1
	fi 
    done
    if [ $? != 0 ]; then exit -1; fi

    echo "$$" > ${TESTDIR}/shellpid

    shopt -s checkwinsize
    clear
    LINES=${LINES:-25}
    COLUMNS=${COLUMNS:-80}

    cleanroot
}

startshell() {
    touch "${TESTDIR}/history"
    pushd "${LIBDIR}" >/dev/null
    rlwrap -H "${TESTDIR}/history" bash -c ". lib/functions.inc ; . lib/lessons/$1 ; startshell_real"    
    popd >/dev/null
}

lastcmd_match() {
    lastcmd=${TESTDIR}/lastcommand
    if egrep "$1" $lastcmd >/dev/null 2>/dev/null; then
	return 0
    else
	return 1
    fi
}

content_match() {
    if egrep "$2" "$1" >/dev/null 2>/dev/null; then
	return 0
    else
	return 1
    fi
}

lastresult() {
    stderr=${TESTDIR}/lastresult.stderr
    stdout=${TESTDIR}/lastresult.stdout
    cat $stderr
    cat $stdout
}

lastresult_match() {
    lastres=${TESTDIR}/lastresult.stdout
    if egrep "$1" $lastres >/dev/null 2>/dev/null; then
	return 0
    else
	return 1
    fi
}

lastresult_compare() {
    ref_stderr=`mktemp`
    ref_stdout=`mktemp`
    bash --norc -c "$1" 2>$ref_stderr 1>$ref_stdout
    lastres=${TESTDIR}/lastresult.stdout
    if diff $ref_stdout $lastres >/dev/null 2>/dev/null; then
	rm -f $ref_stdout $ref_stderr
	return 0
    else
	rm -f $ref_stdout $ref_stderr
	return 1
    fi
}

compare_files() {
    if diff "$1" "$2" >/dev/null 2>/dev/null; then
	return 0
    else
	return 1
    fi
}


sugo() {
    echo -e "$C_YELLOW"
    echo -ne "\nSúgó: "
    lesson_run desc $step
    echo -e "$C_RESET\n"
}

startshell_real() {
    clear
    echo -e "${C_YELLOW}Parancssori üzemmód (kilépés: 'exit' parancs, súgó: 'sugo' parancs)$C_RESET"
    echo -e "\n${C_YELLOW}Színmagyarázat: ${C_RESET}parancsok ${C_YELLOW}információk ${C_GREEN}eredmények ${C_RED}sikertelenség${C_RESET}"
    echo -e "\n${C_YELLOW}Figyelmeztetés: ez az oktató felület nem mindig alkalmas interaktív parancsok"
    echo -e "(pl.: mc, more, less, passwd, read, stb...) futtatására!${C_RESET}"

    echo -e "\n${C_YELLOW}Az aktuális lecke: $lesson_title${C_RESET}"

    echo -e "\n${C_YELLOW}---------------------------------------------------------------------------${C_RESET}"

    cd "${TESTDIR}/root"
    export HOME="${TESTDIR}/root"

    for step in $lesson_steps; do

    sugo

    while true; do
	stderr=${TESTDIR}/lastresult.stderr
    	stdout=${TESTDIR}/lastresult.stdout
	rm -f $stderr
	rm -f $stdout
	read -p "$ " cmd
	lastcmd=${TESTDIR}/lastcommand
        echo "$cmd" > $lastcmd
        if lastcmd_match '^[ \t\n\r]*$'; then
    	    echo -e "$C_GRAY(üres parancs)$C_RESET"
	elif lastcmd_match '^exit$'; then
	    return
	elif lastcmd_match '^sugo$'; then
	    sugo
	elif lastcmd_match '^cd'; then
	    . ${TESTDIR}/lastcommand >/dev/null 2>/dev/null
	    if [ $? = 0 ]; then
		if pwd | grep "${TESTDIR}/root" >/dev/null 2>/dev/null; then		    
        	    echo -e "$C_GREEN(könyvtárváltás sikeres)$C_RESET"	    
		else 
        	    echo -e "$C_RED(könyvtárváltás sikertelen - ne lépjen ki a gyakorló könyvtárból)$C_RESET"	    
		    cd
		fi
	    else 
        	echo -e "$C_RED(könyvtárváltás sikertelen)$C_RESET"	    
	    fi
	    echo -e "${C_YELLOW}Az aktuális könyvtár: "`pwd`"$C_RESET"
	else
	    if lesson_run is_interactive $step; then
    		bash --norc -c "$cmd" 2>$stderr 1>$stdout
		retval=$?
	        cat $stderr
    		cat $stdout
	    else 
		echo -e "$C_GREEN(interaktív parancs futtatása...)$C_RESET"
		echo "" 2>$stderr 1>$stdout
    		bash --norc -c "$cmd"
		retval=$?
	    fi
    	    if [ $retval -eq 0 ]; then
	        echo -e "$C_GREEN(a parancs futtatása sikerült)$C_RESET"
	    else
		echo -e "$C_RED(a parancs futtatása NEM sikerült)$C_RESET"
	    fi
	fi
	if lesson_run check $step; then
	    continue 2
	fi
    done
    
    done
}

menu() {
    while true; do
	whiptailcmd='whiptail --title "Menü" --menu "válassz a menüpontok közül!" '$(($LINES - 4))' '$((COLUMNS - 4))' '$(($LINES - 12))' '
	for lesson in ${LIBDIR}/lib/lessons/*; do
	    . $lesson
	    tag=`basename "$lesson"`
	    whiptailcmd="$whiptailcmd '$tag' '$lesson_title'"
	done
        #echo $menu
        #exit 0
	res=`mktemp`
        if bash -c "$whiptailcmd" 2>$res; then
	    item=`cat $res`
	    rm -f $res
	    echo "menüpont: $item"
	    cleanroot
	    . ${LIBDIR}/lib/lessons/$item
	    lesson_init
	    startshell $item
        else
	    rm -f $res
	    echo "kilépés"
	    exit 0;
        fi
    done
}
